@@ -53,7 +53,8 @@ class AddonTest {
                 "tabHide",
                 "tabs",
                 "topSites",
-                "unlimitedStorage",
+                "trialML",
+                "userScripts",
                 "webNavigation",
                 "devtools",
             ),
@@ -85,7 +86,8 @@ class AddonTest {
             R.string.mozac_feature_addons_permissions_tab_hide_description,
             R.string.mozac_feature_addons_permissions_tabs_description,
             R.string.mozac_feature_addons_permissions_top_sites_description,
-            R.string.mozac_feature_addons_permissions_unlimited_storage_description,
+            R.string.mozac_feature_addons_permissions_trial_ml_description,
+            R.string.mozac_feature_addons_permissions_user_scripts_description,
             R.string.mozac_feature_addons_permissions_web_navigation_description,
             R.string.mozac_feature_addons_permissions_devtools_description,
         ).map { testContext.getString(it) }
@@ -127,6 +129,58 @@ class AddonTest {
         assertTrue(enabledAddon.isEnabled())
     }
 
+    @Test
+    fun `classifyOriginPermissions - normalizes origins and returns HostPermissions`() {
+        val origins = listOf(
+            "https://developer.mozilla.org/",
+            "*://*.developer.mozilla.org/*",
+            "*://developer.mozilla.org/*",
+            "https://wikipedia.org/",
+            "*://*.wikipedia.org/*",
+            "*://wikipedia.org/*",
+        )
+
+        val hostNormalizationResult = Addon.classifyOriginPermissions(origins)
+        assertNotNull(hostNormalizationResult.getOrNull())
+
+        val hostPermissions = hostNormalizationResult.getOrNull()!!
+
+        assertEquals(hostPermissions.sites.size, 2)
+        assertEquals(hostPermissions.wildcards.size, 2)
+
+        val displayDomainList = hostPermissions.wildcards + hostPermissions.sites
+
+        assertEquals(displayDomainList.size, 2)
+
+        assertEquals(displayDomainList.first(), "developer.mozilla.org")
+        assertTrue(displayDomainList.contains("wikipedia.org"))
+    }
+
+    @Test
+    fun `permissionsListContainsAllUrls - true if any permission in the list maps to the All Urls permission`() {
+        val permissionsWithoutAllUrls = listOf("privacy", "tabs")
+        val permissionsWithAllUrls = listOf("privacy", "<all_urls>", "tabs")
+
+        val domainsWithoutAllUrls = listOf(
+            "https://www.mozilla.org",
+            "http://testsite.com",
+            "testing.com",
+            "testing.com/*",
+            "*.testing.com/*",
+        )
+
+        val domainsWithAllUrls = listOf(
+            "testing.com",
+            "*://*/*",
+        )
+
+        assertFalse("Found all_urls permission when none exists", Addon.permissionsListContainsAllUrls(permissionsWithoutAllUrls))
+        assertTrue("Did not find the all_urls permission in the list", Addon.permissionsListContainsAllUrls(permissionsWithAllUrls))
+
+        assertFalse("Found all_urls permission when none exists", Addon.permissionsListContainsAllUrls(domainsWithoutAllUrls))
+        assertTrue("Did not find the all_urls permission in the list", Addon.permissionsListContainsAllUrls(domainsWithAllUrls))
+    }
+
     @Test
     fun `filterTranslations - only keeps specified translations`() {
         val addon = Addon(
@@ -541,4 +595,20 @@ class AddonTest {
         assertNotNull(addonWithInstalledStateIcon.installedState?.icon)
         assertNotNull(addonWithInstalledStateIcon.provideIcon())
     }
+
+    @Test
+    fun `isSoftBlocked - true if installed state disabled status equals to SOFT_BLOCKED and otherwise false`() {
+        val addon = Addon(id = "id")
+        val softBlockedAddon = addon.copy(
+            installedState = Addon.InstalledState(
+                id = "id",
+                version = "1.0",
+                optionsPageUrl = "",
+                disabledReason = Addon.DisabledReason.SOFT_BLOCKED,
+            ),
+        )
+
+        assertFalse(addon.isSoftBlocked())
+        assertTrue(softBlockedAddon.isSoftBlocked())
+    }
 }
